import { Column, Entity, JoinTable, ManyToMany, OneToMany, PrimaryGeneratedColumn } from 'typeorm';
import { CharacterTag } from './character-tag';
import { InteractionTag } from './interaction-tag';
import { Work } from './work';
import { WorkCharacterName } from './work-character-name';
import { WorldCharacter } from './world-character';

@Entity()
export class WorkCharacter implements WorkCharacterEntityInterface {
  @PrimaryGeneratedColumn()
  public readonly id!: number;

  @Column({
    nullable: false,
    default: '',
  })
  public nameCanonical!: string;

  @OneToMany(() => WorkCharacterName, (workCharacterName: WorkCharacterNameEntityInterface) => workCharacterName.entity)
  public names!: Promise<WorkCharacterNameEntityInterface[]>;

  @ManyToMany(() => Work, (work: WorkEntityInterface) => work.workCharacters)
  public works!: Promise<WorkEntityInterface[]>;

  @ManyToMany(() => InteractionTag, (interactionTag: InteractionTagEntityInterface) => interactionTag.subjectCharacters)
  @JoinTable()
  public interactWith!: Promise<InteractionTagEntityInterface[]>;

  @ManyToMany(() => InteractionTag, (interactionTag: InteractionTagEntityInterface) => interactionTag.objectCharacters)
  @JoinTable()
  public interactedBy!: Promise<InteractionTagEntityInterface[]>;

  @OneToMany(() => CharacterTag, (characterTag: CharacterTagEntityInterface) => characterTag.workCharacter)
  public characterTags!: Promise<CharacterTagEntityInterface[]>;

  @ManyToMany(() => WorldCharacter, (worldCharacter: WorldCharacterEntityInterface) => worldCharacter.workCharacters)
  @JoinTable()
  public worldCharacters!: Promise<WorldCharacterEntityInterface[]>;
}
