import path from 'path';
import { injectable } from 'inversify';
import { Service } from '../../core/container';
import { inject } from '../../core/inject';
import { mergeContentSecurityPolicy } from '../session/session-util';
import { UrlAppWindow } from './url-app-window';

@injectable()
export class MainAppWindow extends UrlAppWindow {
  public constructor(@inject(Service.LOGGER) logger: LoggerInterface) {
    super(logger, 'http://localhost:3000', {
      webPreferences: {
        preload: path.resolve('src/preload.js'),
      },
    });
  }

  protected getCsp(): Session.ContentSecurityPolicy {
    return mergeContentSecurityPolicy(super.getCsp(), {
      'script-src-elem': ["'self'"],
      'connect-src': ['ws:'],
    });
  }
}
