import { entityApi } from '../../../services/api';
import { MultiNamedEntityRepository, MultiNamedEntityRepositoryInterface } from '../multi-named-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository } from '../util/update-related';

const authorNameRepositoryPromise = import('./author-name-repository').then((m) => m.authorNameRepository);
const workAuthorRepositoryPromise = import('./work-author-repository').then((m) => m.workAuthorRepository);

export const authorRepository: MultiNamedEntityRepositoryInterface<AuthorSerializedInterface, NameSerializedInterface> =
  new MultiNamedEntityRepository(
    (id) => entityApi.readAuthor(id),
    (partial) => entityApi.createAuthor(partial),
    (id, partial) => entityApi.updateAuthor(id, partial),
    (id) => entityApi.deleteAuthor(id),
    {
      nameCanonical: (updatePartial, key, updated, original): void => {
        fillUpdatePartialPrimitive(updated, original, key, updatePartial);
      },
      names: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      workAuthors: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
    },
    {
      names: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, authorNameRepositoryPromise, updated, original),
      workAuthors: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, workAuthorRepositoryPromise, updated, original),
    },
    authorNameRepositoryPromise,
  );
