import { entityApi } from '../../../services/api';
import { MultiNamedEntityRepository, MultiNamedEntityRepositoryInterface } from '../multi-named-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository } from '../util/update-related';

const authorRoleNameRepositoryPromise = import('./author-role-name-repository').then((m) => m.authorRoleNameRepository);
const workAuthorRepositoryPromise = import('./work-author-repository').then((m) => m.workAuthorRepository);

export const authorRoleRepository: MultiNamedEntityRepositoryInterface<
  AuthorRoleSerializedInterface,
  AuthorRoleNameSerializedInterface
> = new MultiNamedEntityRepository(
  (id) => entityApi.readAuthorRole(id),
  (partial) => entityApi.createAuthorRole(partial),
  (id, partial) => entityApi.updateAuthorRole(id, partial),
  (id) => entityApi.deleteAuthorRole(id),
  {
    nameCanonical: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    description: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    names: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
    workAuthors: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
  },
  {
    names: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, authorRoleNameRepositoryPromise, updated, original),
    workAuthors: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, workAuthorRepositoryPromise, updated, original),
  },
  authorRoleNameRepositoryPromise,
);
