import { entityApi } from '../../../services/api';
import { MultiNamedEntityRepository, MultiNamedEntityRepositoryInterface } from '../multi-named-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository } from '../util/update-related';

const collectionNameRepositoryPromise = import('./collection-name-repository').then((m) => m.collectionNameRepository);
const collectionPartRepositoryPromise = import('./collection-part-repository').then((m) => m.collectionPartRepository);

export const collectionRepository: MultiNamedEntityRepositoryInterface<
  CollectionSerializedInterface,
  CollectionNameSerializedInterface
> = new MultiNamedEntityRepository(
  (id) => entityApi.readCollection(id),
  (partial) => entityApi.createCollection(partial),
  (id, partial) => entityApi.updateCollection(id, partial),
  (id) => entityApi.deleteCollection(id),
  {
    nameCanonical: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    names: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
    parts: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
  },
  {
    names: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, collectionNameRepositoryPromise, updated, original),
    parts: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, collectionPartRepositoryPromise, updated, original),
  },
  collectionNameRepositoryPromise,
);
