import { entityApi } from '../../../services/api';
import { EditableEntityRepository } from '../editable-entity-repository';
import type { EditableEntityRepositoryInterface } from '../editable-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository, updateSingleRelatedRepository } from '../util/update-related';

const sourceRepositoryPromise = import('./source-repository').then((m) => m.sourceRepository);
const workRepositoryPromise = import('./work-repository').then((m) => m.workRepository);

export const copyRepository: EditableEntityRepositoryInterface<CopySerializedInterface> = new EditableEntityRepository(
  (id) => entityApi.readCopy(id),
  (partial) => entityApi.createCopy(partial),
  (id, partial) => entityApi.updateCopy(id, partial),
  (id) => entityApi.deleteCopy(id),
  {
    ranking: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    original: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    sources: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
  },
  {
    original: (updatedPartial, key, updated, original): Promise<void> =>
      updateSingleRelatedRepository(updatedPartial, key, workRepositoryPromise, updated, original),
    sources: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, sourceRepositoryPromise, updated, original),
  },
);
