import { entityApi } from '../../../services/api';
import { EditableEntityRepository } from '../editable-entity-repository';
import type { EditableEntityRepositoryInterface } from '../editable-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository, updateSingleRelatedRepository } from '../util/update-related';

const tagRepositoryPromise = import('./tag-repository').then((m) => m.tagRepository);
const workCharacterRepositoryPromise = import('./work-character-repository').then((m) => m.workCharacterRepository);

export const interactionTagRepository: EditableEntityRepositoryInterface<InteractionTagSerializedInterface> =
  new EditableEntityRepository(
    (id) => entityApi.readInteractionTag(id),
    (partial) => entityApi.createInteractionTag(partial),
    (id, partial) => entityApi.updateInteractionTag(id, partial),
    (id) => entityApi.deleteInteractionTag(id),
    {
      weight: (updatePartial, key, updated, original): void => {
        fillUpdatePartialPrimitive(updated, original, key, updatePartial);
      },
      tag: (updatePartial, key, updated, original): void => {
        fillUpdatePartialPrimitive(updated, original, key, updatePartial);
      },
      objectCharacters: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      subjectCharacters: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
    },
    {
      tag: (updatedPartial, key, updated, original): Promise<void> =>
        updateSingleRelatedRepository(updatedPartial, key, tagRepositoryPromise, updated, original),
      objectCharacters: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, workCharacterRepositoryPromise, updated, original),
      subjectCharacters: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, workCharacterRepositoryPromise, updated, original),
    },
  );
