import { entityApi } from '../../../services/api';
import { MultiNamedEntityRepository, MultiNamedEntityRepositoryInterface } from '../multi-named-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository } from '../util/update-related';

const siteNameRepositoryPromise = import('./site-name-repository').then((m) => m.siteNameRepository);
const sourceRepositoryPromise = import('./source-repository').then((m) => m.sourceRepository);

export const siteRepository: MultiNamedEntityRepositoryInterface<SiteSerializedInterface, SiteNameSerializedInterface> =
  new MultiNamedEntityRepository(
    (id) => entityApi.readSite(id),
    (partial) => entityApi.createSite(partial),
    (id, partial) => entityApi.updateSite(id, partial),
    (id) => entityApi.deleteSite(id),
    {
      nameCanonical: (updatePartial, key, updated, original): void => {
        fillUpdatePartialPrimitive(updated, original, key, updatePartial);
      },
      names: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      sources: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
    },
    {
      names: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, siteNameRepositoryPromise, updated, original),
      sources: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, sourceRepositoryPromise, updated, original),
    },
    siteNameRepositoryPromise,
  );
