import { entityApi } from '../../../services/api';
import { MultiNamedEntityRepository, MultiNamedEntityRepositoryInterface } from '../multi-named-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository } from '../util/update-related';

const characterTagRepositoryPromise = import('./character-tag-repository').then((m) => m.characterTagRepository);
const interactionTagRepositoryPromise = import('./interaction-tag-repository').then((m) => m.interactionTagRepository);
const tagNameRepositoryPromise = import('./tag-name-repository').then((m) => m.tagNameRepository);
const workTagRepositoryPromise = import('./work-tag-repository').then((m) => m.workTagRepository);

export const tagRepository: MultiNamedEntityRepositoryInterface<TagSerializedInterface, TagNameSerializedInterface> =
  new MultiNamedEntityRepository(
    (id) => entityApi.readTag(id),
    (partial) => entityApi.createTag(partial),
    (id, partial) => entityApi.updateTag(id, partial),
    (id) => entityApi.deleteTag(id),
    {
      nameCanonical: (updatePartial, key, updated, original): void => {
        fillUpdatePartialPrimitive(updated, original, key, updatePartial);
      },
      description: (updatePartial, key, updated, original): void => {
        fillUpdatePartialPrimitive(updated, original, key, updatePartial);
      },
      names: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      parents: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      children: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      workTags: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      interactionTags: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
      characterTags: (updatePartial, key, updated, original): void => {
        fillUpdatePartialArray(updated, original, key, updatePartial);
      },
    },
    {
      names: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, tagNameRepositoryPromise, updated, original),
      parents: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, Promise.resolve(tagRepository), updated, original),
      children: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, Promise.resolve(tagRepository), updated, original),
      workTags: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, workTagRepositoryPromise, updated, original),
      interactionTags: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, interactionTagRepositoryPromise, updated, original),
      characterTags: (updatedPartial, key, updated, original): Promise<void> =>
        updateMultiRelatedRepository(updatedPartial, key, characterTagRepositoryPromise, updated, original),
    },
    tagNameRepositoryPromise,
  );
