import { entityApi } from '../../../services/api';
import { EditableEntityRepository } from '../editable-entity-repository';
import { fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateSingleRelatedRepository } from '../util/update-related';

const tagRepositoryPromise = import('./tag-repository').then((m) => m.tagRepository);
const workRepositoryPromise = import('./work-repository').then((m) => m.workRepository);

export const workTagRepository: EditableEntityRepository<WorkTagSerializedInterface> = new EditableEntityRepository(
  (id) => entityApi.readWorkTag(id),
  (partial) => entityApi.createWorkTag(partial),
  (id, partial) => entityApi.updateWorkTag(id, partial),
  (id) => entityApi.deleteWorkTag(id),
  {
    weight: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    work: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    tag: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
  },
  {
    work: (updatedPartial, key, updated, original): Promise<void> =>
      updateSingleRelatedRepository(updatedPartial, key, workRepositoryPromise, updated, original),
    tag: (updatedPartial, key, updated, original): Promise<void> =>
      updateSingleRelatedRepository(updatedPartial, key, tagRepositoryPromise, updated, original),
  },
);
