import { entityApi } from '../../../services/api';
import { MultiNamedEntityRepository, MultiNamedEntityRepositoryInterface } from '../multi-named-entity-repository';
import { fillUpdatePartialArray, fillUpdatePartialPrimitive } from '../util/fill-update-partial';
import { updateMultiRelatedRepository } from '../util/update-related';

const workRepositoryPromise = import('./work-repository').then((m) => m.workRepository);
const worldCharacterRepositoryPromise = import('./world-character-repository').then((m) => m.worldCharacterRepository);
const worldNameRepositoryPromise = import('./world-name-repository').then((m) => m.worldNameRepository);

export const worldRepository: MultiNamedEntityRepositoryInterface<
  WorldSerializedInterface,
  WorldNameSerializedInterface
> = new MultiNamedEntityRepository(
  (id) => entityApi.readWorld(id),
  (partial) => entityApi.createWorld(partial),
  (id, partial) => entityApi.updateWorld(id, partial),
  (id) => entityApi.deleteWorld(id),
  {
    nameCanonical: (updatePartial, key, updated, original): void => {
      fillUpdatePartialPrimitive(updated, original, key, updatePartial);
    },
    names: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
    works: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
    worldCharacters: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
    parents: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
    children: (updatePartial, key, updated, original): void => {
      fillUpdatePartialArray(updated, original, key, updatePartial);
    },
  },
  {
    names: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, worldNameRepositoryPromise, updated, original),
    works: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, workRepositoryPromise, updated, original),
    worldCharacters: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, worldCharacterRepositoryPromise, updated, original),
    parents: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, Promise.resolve(worldRepository), updated, original),
    children: (updatedPartial, key, updated, original): Promise<void> =>
      updateMultiRelatedRepository(updatedPartial, key, Promise.resolve(worldRepository), updated, original),
  },
  worldNameRepositoryPromise,
);
