import {
  EditableEntityRepository,
  RelatedRepositoryUpdaters,
  UpdatePartialFillers,
} from './editable-entity-repository';
import type { MultiNamedEntityRepositoryInterface } from './multi-named-entity-repository';
import { fillUpdatePartialPrimitive } from './util/fill-update-partial';
import { updateSingleRelatedRepository } from './util/update-related';

/**
 * entity repository for {@link NameInterface}
 */
export class NameEntityRepository<
  Serialized extends NameSerializedInterface,
  MultiNamedSerialized extends MultiNamedInterface,
> extends EditableEntityRepository<Serialized> {
  public constructor(
    apiRead: (identifier: number) => Promise<Serialized>,
    apiCreate: (partial: Partial<Serialized>) => Promise<Serialized>,
    apiUpdate: (identifier: number, partial: Partial<Serialized>) => Promise<Serialized>,
    apiDelete: (identifier: number) => Promise<void>,
    multiNamedEntityRepositoryPromise: Promise<MultiNamedEntityRepositoryInterface<MultiNamedSerialized, Serialized>>,
  ) {
    super(
      apiRead,
      apiCreate,
      apiUpdate,
      apiDelete,
      {
        name: (updatePartial, key, updated, original) => {
          fillUpdatePartialPrimitive(updated, original, key, updatePartial);
        },
        entity: (updatePartial, key, updated, original) => {
          fillUpdatePartialPrimitive(updated, original, key, updatePartial);
        },
      } as UpdatePartialFillers<Serialized>,
      {
        entity: (updatedPartial, key, updated, original) =>
          updateSingleRelatedRepository(updatedPartial, key, multiNamedEntityRepositoryPromise, updated, original),
      } as RelatedRepositoryUpdaters<Serialized>,
    );
  }
}
