import { equalArray } from '../../../services/util/compare';

/**
 * fills the update partial when the specified property is different in the updated entity
 *
 * @param updated - the updated entity
 * @param original - the original entity
 * @param key - the key of the entity, specifying the property to compare
 * @param updatePartial - the update partial to fill
 */
export function fillUpdatePartialPrimitive<K extends string, T extends boolean | number | string | null>(
  updated: { [x in K]: T },
  original: { [x in K]: T },
  key: K,
  updatePartial: Partial<{ [x in K]: T }>,
): void {
  if (updated[key] !== original[key]) {
    updatePartial[key] = updated[key];
  }
}

/**
 * fills the update partial when the specified array property is different in the updated entity
 *
 * @param updated - the updated entity
 * @param original - the original entity
 * @param key - the key of the entity, specifying the property to compare
 * @param updatePartial - the update partial to fill
 */
export function fillUpdatePartialArray<K extends string, Id extends Identifier = number>(
  updated: { [x in K]: Id[] },
  original: { [x in K]: Id[] },
  key: K,
  updatePartial: Partial<{ [x in K]: Id[] }>,
): void {
  if (!equalArray(updated[key], original[key])) {
    updatePartial[key] = updated[key];
  }
}
