import { symmetricDifference } from '../../../services/util/set';
import type { EntityRepositoryInterface } from '../entity-repository';

export async function updateSingleRelatedRepository<K extends string, Id extends Identifier = number>(
  updatePartial: Partial<{ [x in K]: Id | null }>,
  key: K,
  repositoryPromise: Promise<EntityRepositoryInterface<IdentifiableInterface<Id>, Id>>,
  updated: { [x in K]: Id | null } | undefined,
  original: { [x in K]: Id | null } | undefined,
): Promise<void> {
  const repository = await repositoryPromise;
  if (updatePartial[key] === undefined) {
    return Promise.resolve();
  }
  const promises = [];
  if (updated?.[key]) {
    promises.push(repository.refresh(updated[key] as Id));
  }
  if (original?.[key]) {
    promises.push(repository.refresh(original[key] as Id));
  }
  return Promise.all(promises).then();
}

export async function updateMultiRelatedRepository<K extends string, Id extends Identifier = number>(
  updatePartial: Partial<{ [x in K]: Id[] }>,
  key: K,
  repositoryPromise: Promise<EntityRepositoryInterface<IdentifiableInterface<Id>, Id>>,
  updated: { [x in K]: Id[] } | undefined,
  original: { [x in K]: Id[] } | undefined,
): Promise<void> {
  const repository = await repositoryPromise;
  if (updatePartial[key] === undefined) {
    return Promise.resolve();
  }
  let updateIds: Id[] = [];
  if (updated && original) {
    updateIds = Array.from(symmetricDifference(updated[key], original[key]));
  } else if (updated) {
    updateIds = updated[key];
  } else if (original) {
    updateIds = original[key];
  }
  return Promise.all(updateIds.map((id) => repository.refresh(id))).then();
}
