/**
 * This entity describes a character in a work.
 * The character can be original or based on one or more existing characters.
 */
interface WorkCharacterInterface extends IdentifiableInterface, MultiNamedInterface {
  names: Promise<WorkCharacterNameInterface[]> | number[];

  /**
   * the works the character is a part of
   * one work character can be part of multiple works because of series
   */
  works: Promise<WorkInterface[]> | number[];

  /**
   * interaction with other characters as actor
   */
  interactWith: Promise<InteractionTagInterface[]> | number[];

  /**
   * interaction with other characters as receiver
   */
  interactedBy: Promise<InteractionTagInterface[]> | number[];

  /**
   * tags connected to the character
   */
  characterTags: Promise<CharacterTagInterface[]> | number[];

  /**
   * existing characters character is based on
   */
  worldCharacters: Promise<WorldCharacterInterface[]> | number[];
}
